package com.factory.controller.web;

import java.util.List;

import com.factory.user.entity.vo.res.FileRes;
import com.factory.web.entity.fa.req.*;
import com.factory.web.entity.vo.req.exportInitialPlanDetailsReq;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.fa.res.FaCalendarRes;
import com.factory.web.entity.fa.res.FaCheckRecordListRes;
import com.factory.web.entity.fa.res.FaItemListForPadRes;
import com.factory.web.entity.fa.res.FaItemNoGoListRes;
import com.factory.web.entity.fa.res.FaNoGoTaskAapprovalRes;
import com.factory.web.entity.fa.res.FaPlanListForPadRes;
import com.factory.web.entity.fa.res.FaPlanListRes;
import com.factory.web.entity.fa.res.FaPlanRes;
import com.factory.web.entity.fa.res.FaWorkshopNoGoListRes;
import com.factory.web.entity.fa.res.UserListRes;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.req.PmPlanPauseFlagReq;
import com.factory.web.entity.pm.req.SearchPmPlanReq;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.pm.PreventMistakeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import lombok.extern.slf4j.Slf4j;

/**
 * 首件计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
@Api(tags = "首件")
@RestController
@RequestMapping("/benteler/fa")
@Slf4j
public class FirstArticleController {
	@Autowired
	private FirstArticleService firstArticleService;

	@Autowired
	private PreventMistakeService preventMistakeService;

	@ApiOperation(value = "新建首件计划")
	@PostMapping("/addFaPlan")
	@ApiOperationSupport(order = 1)
	public ResponseBean addFaPlan(@RequestBody AddFaPlanReq req) {
		return firstArticleService.addFaPlan(req);
	}

	@ApiOperation(value = "取得首件计划信息")
	@ApiOperationSupport(order = 2)
	@PostMapping("/getFaPlan")
	public ResponseBean<FaPlanRes> getFaPlan(@RequestBody SearchFaPlanReq req) {
		FaPlanRes faPlanRes = firstArticleService.getFaPlan(req);
		return ResponseBeanBuilder.ok(faPlanRes);
	}

	@ApiOperation(value = "修改首件计划")
	@PostMapping("/editFaPlan")
	@ApiOperationSupport(order = 3)
	public ResponseBean editFaPlan(@RequestBody EditFaPlanReq req) {
		return firstArticleService.editFaPlan(req);
	}

	@ApiOperation(value = "禁用解禁首件计划")
	@PostMapping("/disableFaPlan")
	@ApiOperationSupport(order = 4)
	public ResponseBean disableFaPlan(@RequestBody PmPlanPauseFlagReq req) {
		return preventMistakeService.disablePmPlan(req);
	}

	@ApiOperation(value = "取得Web端首件计划点检列表")
	@ApiOperationSupport(order = 5)
	@PostMapping("/getFaPlanCheckList")
	public ResponseBean<PageBean<FaPlanListRes>> getFaPlanCheckList(@RequestBody SearchFaPlanCheckListReq req) {
		return firstArticleService.getFaPlanCheckList(req);
	}

	@ApiOperation(value = "取得Pad端首件计划列表")
	@ApiOperationSupport(order = 6)
	@PostMapping("/getFaPlanListForPad")
	public ResponseBean<List<FaPlanListForPadRes>> getFaPlanListForPad(@RequestBody SearchFaPlanListForPadReq req) {
		return firstArticleService.getFaPlanListForPad(req);
	}

	@ApiOperation(value = "取得Pad端工作站列表")
	@ApiOperationSupport(order = 7)
	@PostMapping("/getFaWorkshopListForPad")
	public ResponseBean<List<FirstArticleWorkshop>> getFaWorkshopListForPad(@RequestBody SearchFaWorkshopListForPadReq req) {
		return firstArticleService.getFaWorkshopListForPad(req);
	}

	@ApiOperation(value = "取得Pad端巡检项目列表")
	@ApiOperationSupport(order = 8)
	@PostMapping("/getFaItemListForPad")
	public ResponseBean<List<FaItemListForPadRes>> getFaItemListForPad(@RequestBody SearchFaItemListForPadReq req) {
		return firstArticleService.getFaItemListForPad(req);
	}

	@ApiOperation(value = "添加巡检项目检查结果")
	@PostMapping("/addFaItemResult")
	@ApiOperationSupport(order = 9)
	public ResponseBean addFaItemResult(@RequestBody AddFaItemResultListReq req) {
		return firstArticleService.addFaItemResult(req);
	}

	@ApiOperation(value = "任务检查完后提交")
	@PostMapping("/submitFaTask")
	@ApiOperationSupport(order = 10)
	public ResponseBean submitFaTask(@RequestBody SubmitFaTaskReq req) {
		return firstArticleService.submitFaTask(req);
	}

	@ApiOperation(value = "取得不放行任务对应的工作站")
	@ApiOperationSupport(order = 11)
	@PostMapping("/getFaWorkshopNoGoList")
	public ResponseBean<List<FaWorkshopNoGoListRes>> getFaWorkshopNoGoList(@RequestBody SearchFaWorkshopNoGoListReq req) {
		return firstArticleService.getFaWorkshopNoGoList(req);
	}

	@ApiOperation(value = "取得不放行任务对应的点检项目")
	@ApiOperationSupport(order = 12)
	@PostMapping("/getFaItemNoGoList")
	public ResponseBean<List<FaItemNoGoListRes>> getFaItemNoGoList(@RequestBody SearchFaItemNoGoListReq req) {
		return firstArticleService.getFaItemNoGoList(req);
	}

	@ApiOperation(value = "修改巡检项目检查结果")
	@PostMapping("/editFaItemResult")
	@ApiOperationSupport(order = 13)
	public ResponseBean editFaItemResult(@RequestBody EditFaItemResultListReq req) {
		return firstArticleService.editFaItemResult(req);
	}

	@ApiOperation(value = "通过计划ID取得表头")
	@ApiOperationSupport(order = 14)
	@PostMapping("/getFaTitle")
	public ResponseBean<FirstArticleTitle> getFaTitle(@RequestBody SearchFaPlanReq req) {
		FirstArticleTitle firstArticleTitle = firstArticleService.getFaTitle(req.getBentelerPlanId());
		return ResponseBeanBuilder.ok(firstArticleTitle);
	}

	@ApiOperation(value = "取得不放行任务对应的质量经理审批信息")
	@ApiOperationSupport(order = 15)
	@PostMapping("/getFaNoGoTaskAapproval")
	public ResponseBean<FaNoGoTaskAapprovalRes> getFaNoGoTaskAapproval(@RequestBody SearchFaTaskReq req) {
		return firstArticleService.getFaNoGoTaskAapproval(req);
	}

	@ApiOperation(value = "取得计划审批信息")
	@ApiOperationSupport(order = 16)
	@PostMapping("/getFaPlanApprovalList")
	public ResponseBean<List<OnsiteApprovalPlan>> getFaPlanApprovalList(@RequestBody SearchPmPlanReq req) {
		return preventMistakeService.getPmPlanApprovalList(req);
	}

	@ApiOperation(value = "取得所有计划(首件、防错)")
	@ApiOperationSupport(order = 17)
	@PostMapping("/getALLPlanList")
	public ResponseBean<List<BentelerPlan>> getALLPlanList(@RequestBody SearchALLPlanListReq req) {
		return firstArticleService.getALLPlanList(req);
	}

	@ApiOperation(value = "取得某个计划生成的历史计划信息(首件、防错)")
	@ApiOperationSupport(order = 100)
	@PostMapping("/getHisPlanList")
	public ResponseBean<List<BentelerPlan>> getHisPlanList(@RequestBody SearchHisPlanListReq req) {
		return firstArticleService.getHisPlanList(req);
	}

	@ApiOperation(value = "检查记录列表")
	@ApiOperationSupport(order = 18)
	@PostMapping("/getFaCheckRecordList")
	public ResponseBean<FaCheckRecordListRes> getFaCheckRecordList(@RequestBody SearchFaCheckRecordListReq req) {
		return firstArticleService.getFaCheckRecordList(req);
	}
	@ApiOperation(value = "导出检查记录列表")
	@ApiOperationSupport(order = 18)
	@PostMapping("/exportFaCheckRecordList")
	public ResponseBean<FileRes> exportFaCheckRecordList(@RequestBody SearchFaCheckRecordListReq req) {
		return firstArticleService.exportFaCheckRecordList(req);
	}

	@ApiOperation(value = "首件导出计划信息Excel")
	@PostMapping("/exportInitialPlanDetails")
	public ResponseBean<FileRes> exportInitialPlanDetails(@RequestBody exportInitialPlanDetailsReq req) {
		return firstArticleService.exportInitialPlanDetails(req);
	}

//	@ApiOperation(value = "判断检验结果是否填写完成")
//	@PostMapping("/getFaAllCheckFlag")
//	@ApiOperationSupport(order = 19)
//	public ResponseBean<FaAllCheckFlagRes> getFaAllCheckFlag(@RequestBody SearchFaAllCheckFlagReq req) {
//		return ResponseBeanBuilder.ok(firstArticleService.getFaAllCheckFlag(req));
//	}

	@ApiOperation(value = "取得用户列表")
	@PostMapping("/getUserList")
	@ApiOperationSupport(order = 20)
	public ResponseBean<List<UserListRes>> getUserList() {
		return ResponseBeanBuilder.ok(firstArticleService.getUserList());
	}

	@ApiOperation(value = "取得点检人和点检日期")
	@PostMapping("/getFaCalendarList")
	@ApiOperationSupport(order = 21)
	public ResponseBean<List<FaCalendarRes>> getFaCalendarList(@RequestBody SearchFaCalendarReq req) {
		return ResponseBeanBuilder.ok(firstArticleService.getFaCalendarList(req));
	}

}
