package com.factory.web.service.impl.pm;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.enumPo.PlanType;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.OnsiteApprovalTask;
import com.factory.web.entity.pm.OnsiteCalendar;
import com.factory.web.entity.pm.OnsiteItem;
import com.factory.web.entity.pm.OnsiteItemResult;
import com.factory.web.entity.pm.OnsitePlanHisBlob;
import com.factory.web.entity.pm.OnsiteTask;
import com.factory.web.entity.pm.OnsiteTaskHisBlob;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteTitleBlob;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.entity.pm.PlanBind;
import com.factory.web.entity.pm.req.AddItemResultListReq;
import com.factory.web.entity.pm.req.AddItemResultReq;
import com.factory.web.entity.pm.req.AddPmPlanReq;
import com.factory.web.entity.pm.req.EditItemResultListReq;
import com.factory.web.entity.pm.req.EditItemResultReq;
import com.factory.web.entity.pm.req.EditPmPlanReq;
import com.factory.web.entity.pm.req.OnsiteCalendarReq;
import com.factory.web.entity.pm.req.OnsiteItemReq;
import com.factory.web.entity.pm.req.OnsiteWorkshopReq;
import com.factory.web.entity.pm.req.PmPlanPauseFlagReq;
import com.factory.web.entity.pm.req.SearchAbnormalListReq;
import com.factory.web.entity.pm.req.SearchAbnormalReq;
import com.factory.web.entity.pm.req.SearchAllWorkshopReq;
import com.factory.web.entity.pm.req.SearchApprovalUserReq;
import com.factory.web.entity.pm.req.SearchCheckRecordReq;
import com.factory.web.entity.pm.req.SearchItemListReq;
import com.factory.web.entity.pm.req.SearchPmAllCheckFlagReq;
import com.factory.web.entity.pm.req.SearchPmCheckRecordListReq;
import com.factory.web.entity.pm.req.SearchPmItemNoGoListReq;
import com.factory.web.entity.pm.req.SearchPmPlanCheckListReq;
import com.factory.web.entity.pm.req.SearchPmPlanListForPadReq;
import com.factory.web.entity.pm.req.SearchPmPlanListReq;
import com.factory.web.entity.pm.req.SearchPmPlanReq;
import com.factory.web.entity.pm.req.SearchPmWorkshopNoGoListReq;
import com.factory.web.entity.pm.req.SearchPmWsAllCheckFlagReq;
import com.factory.web.entity.pm.req.SearchTaskReq;
import com.factory.web.entity.pm.req.SearchWorkshopReq;
import com.factory.web.entity.pm.res.AbnormalItemRes;
import com.factory.web.entity.pm.res.AbnormalItemResultRes;
import com.factory.web.entity.pm.res.AbnormalTaskInfoRes;
import com.factory.web.entity.pm.res.AbnormalWorkshopRes;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.entity.pm.res.PmAllCheckFlagRes;
import com.factory.web.entity.pm.res.PmAllWorkshopListRes;
import com.factory.web.entity.pm.res.PmCheckListRes;
import com.factory.web.entity.pm.res.PmCheckRecordItemListRes;
import com.factory.web.entity.pm.res.PmCheckRecordListRes;
import com.factory.web.entity.pm.res.PmCheckRecordRes;
import com.factory.web.entity.pm.res.PmItemNoGoListRes;
import com.factory.web.entity.pm.res.PmNoGoTaskAapprovalRes;
import com.factory.web.entity.pm.res.PmPlanCheckListRes;
import com.factory.web.entity.pm.res.PmPlanListForPadRes;
import com.factory.web.entity.pm.res.PmPlanListRes;
import com.factory.web.entity.pm.res.PmPlanRes;
import com.factory.web.entity.pm.res.PmTaskListRes;
import com.factory.web.entity.pm.res.PmTaskRes;
import com.factory.web.entity.pm.res.PmWorkshopNoGoListRes;
import com.factory.web.mapper.pm.PreventMistakeMapper;
import com.factory.web.service.BentelerPlanService;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.fa.FirstArticleWorkshopService;
import com.factory.web.service.pm.OnsiteApprovalPlanService;
import com.factory.web.service.pm.OnsiteApprovalTaskService;
import com.factory.web.service.pm.OnsiteCalendarService;
import com.factory.web.service.pm.OnsiteItemResultService;
import com.factory.web.service.pm.OnsiteItemService;
import com.factory.web.service.pm.OnsitePlanHisBlobService;
import com.factory.web.service.pm.OnsiteTaskHisBlobService;
import com.factory.web.service.pm.OnsiteTaskService;
import com.factory.web.service.pm.OnsiteTitleBlobService;
import com.factory.web.service.pm.OnsiteWorkshopService;
import com.factory.web.service.pm.PlanBindService;
import com.factory.web.service.pm.PreventMistakeService;


/**
 * 防错计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
@Service("preventMistakeService")
@Transactional(rollbackFor = Exception.class)
public class PreventMistakeServiceImpl extends ServiceImpl<PreventMistakeMapper, BentelerPlan> implements PreventMistakeService {

	@Autowired
	private PreventMistakeMapper preventMistakeMapper;

	@Autowired
	private BentelerPlanService planService;

	@Autowired
	private OnsiteCalendarService onsiteCalendarService;

	@Autowired
	private OnsiteTitleBlobService onsiteTitleBlobService;

	@Autowired
	private OnsiteWorkshopService onsiteWorkshopService;

	@Autowired
	private OnsiteItemService onsiteItemService;

	@Autowired
	private OnsiteItemResultService onsiteItemResultService;

	@Autowired
	private OnsiteTaskService onsiteTaskService;

	@Autowired
	private OnsiteApprovalTaskService onsiteApprovalTaskService;

	@Autowired
	private OnsiteApprovalPlanService onsiteApprovalPlanService;

	@Autowired
	private OnsitePlanHisBlobService onsitePlanHisBlobService;

	@Autowired
	private OnsiteTaskHisBlobService onsiteTaskHisBlobService;

	@Autowired
	private FirstArticleWorkshopService firstArticleWorkshopService;
	
	@Autowired
	private FirstArticleService firstArticleService;
	
	@Autowired
	private PlanBindService planBindService;
	

	/**
	 * 新建防错计划
	 *
	 * @param req
	 * @return
	 */
	@Override
	public ResponseBean addPmPlan(AddPmPlanReq req) {
		/* 校验计划名称是否存在 */
		if (this.checkPlanNameExist(req.getBentelerPlanName())) {
			return ResponseBeanBuilder.fail("计划名称已存在");
		}

		/* 添加防错计划 */
		BentelerPlan plan = BentelerPlan.builder()
				.planType(PlanType.防错.value)
				.name(req.getBentelerPlanName())
				.pauseFlag(false)
				.effectiveDate(req.getEffectiveDate())
				.build();
		planService.save(plan);

		/* 添加日周月点检规则 */
		List<OnsiteCalendarReq> onsiteCalendarList = req.getOnsiteCalendarList();
		for (OnsiteCalendarReq onsiteCalendarReq : onsiteCalendarList) {
			OnsiteCalendar onsiteCalendar = OnsiteCalendar.builder()
					.bentelerPlanId(plan.getId())
					.startDay(onsiteCalendarReq.getStartDay())
					.endDay(onsiteCalendarReq.getEndDay())
					.dayValidate(onsiteCalendarReq.getDayValidate())
					.dayOuttime(onsiteCalendarReq.getDayOuttime())
					.weekDay(onsiteCalendarReq.getWeekDay())
					.monthDay(onsiteCalendarReq.getMonthDay())
					.month(onsiteCalendarReq.getMonth())
					.userId(onsiteCalendarReq.getUserId())
					.typeFlag(onsiteCalendarReq.getTypeFlag())
					.shift(onsiteCalendarReq.getShift())
					.build();
			onsiteCalendarService.save(onsiteCalendar);
		}

		/* 添加表头信息 */
		try {
			OnsiteTitleBlob onsiteTitleBlob = OnsiteTitleBlob.builder()
					.bentelerPlanId(plan.getId())
					.titleContent(this.objectToByteArray(req.getOnsiteTitle()))
					.build();
			onsiteTitleBlobService.save(onsiteTitleBlob);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/* 添加工作站信息 */
		List<OnsiteWorkshopReq> onsiteWorkshopList = req.getOnsiteWorkshopList();
		for (OnsiteWorkshopReq onsiteWorkshopReq : onsiteWorkshopList) {
//			if (this.checkWsNameExist(onsiteWorkshopReq.getWsName())) {
//				return ResponseBeanBuilder.fail("工作站名称" + onsiteWorkshopReq.getWsName() + "已存在");
//			}

			OnsiteWorkshop onsiteWorkshop = OnsiteWorkshop.builder()
					.bentelerPlanId(plan.getId())
					.wsName(onsiteWorkshopReq.getWsName())
					.build();
			onsiteWorkshopService.save(onsiteWorkshop);

			/* 添加巡检项目信息 */
			List<OnsiteItemReq> onsiteItemList = onsiteWorkshopReq.getOnsiteItemList();
			for (OnsiteItemReq onsiteItemReq : onsiteItemList) {
				OnsiteItem onsiteItem = OnsiteItem.builder()
						.workshopId(onsiteWorkshop.getId())
						.itemContent(onsiteItemReq.getItemContent())
						.typeFlag(onsiteItemReq.getTypeFlag())
						.inspectionFlag(onsiteItemReq.getInspectionFlag())
						.build();
				onsiteItemService.save(onsiteItem);
			}
		}

		/* 取得审批人 approvalLevel=1级审批 type=1计划审批 */
		ApprovalUserRes approvalUserRes = this.getApprovalUserInfo(1, 1);
		if (approvalUserRes != null) {
			/* 添加计划一级审批 */
			OnsiteApprovalPlan onsiteApprovalPlan = OnsiteApprovalPlan.builder()
					.bentelerPlanId(plan.getId())
					.approvalLevel(1) // 1级审批
					.approvalUserId(approvalUserRes.getUserId())
					.status(0) // 0未审批
					.build();
			onsiteApprovalPlanService.save(onsiteApprovalPlan);
		}

		return ResponseBeanBuilder.ok();
	}

	/**
	 * 校验计划名是否存在
	 *
	 * @param name
	 * @return true存在，false不存在
	 */
	@Override
	public boolean checkPlanNameExist(String name) {
		int count = planService.count(
				Wrappers.<BentelerPlan> lambdaQuery().eq(BentelerPlan::getName, name));
		if (count > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 校验工作站名是否存在
	 *
	 * @param wsName
	 * @return true存在，false不存在
	 */
	@Override
	public boolean checkWsNameExist(String wsName) {
		int count = onsiteWorkshopService.count(
				Wrappers.<OnsiteWorkshop> lambdaQuery().eq(OnsiteWorkshop::getWsName, wsName));
		if (count > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
     * 取得防错计划点检列表
     * @param req
     * @return 防错计划点检列表
     */
	@Override
    public ResponseBean<PageBean<PmPlanCheckListRes>> getPmPlanCheckList(SearchPmPlanCheckListReq req) {
    	Page<PmPlanCheckListRes> page = new Page<>(req.getPage(), req.getSize());
		Page<PmPlanCheckListRes> listResPage = preventMistakeMapper.getPmPlanCheckList(page, req);
		return ResponseBeanBuilder.ok(listResPage, Function.identity());
    }

	/**
	 * 取得防错计划列表
	 *
	 * @param req
	 * @return 防错计划列表
	 */
	@Override
	public ResponseBean<PageBean<PmPlanListRes>> getPmPlanList(SearchPmPlanListReq req) {
		Page<PmPlanListRes> page = new Page<>(req.getPage(), req.getSize());
		Page<PmPlanListRes> listResPage = preventMistakeMapper.getPmPlanList(page, req);

		SearchPmPlanReq searchPmPlanReq = null;
		for (PmPlanListRes pmPlan : listResPage.getRecords()) {
			searchPmPlanReq = new SearchPmPlanReq();
			searchPmPlanReq.setBentelerPlanId(pmPlan.getBentelerPlanId());
			OnsiteApprovalPlan latestApprovalInfo = this.getLatestApprovalInfo(searchPmPlanReq);

			pmPlan.setApprovalLevel(latestApprovalInfo.getApprovalLevel());
			pmPlan.setApprovalUserId(latestApprovalInfo.getApprovalUserId());
			pmPlan.setStatus(latestApprovalInfo.getStatus());
			pmPlan.setReason(latestApprovalInfo.getReason());
			pmPlan.setUpdatedTime(latestApprovalInfo.getUpdatedTime());
		}

//		// TODO 测一下status值是都设置成功
//		SearchPmPlanReq searchPmPlanReq = null;
//		for (PmPlanListRes pmPlan : listResPage.getRecords()) {
//			searchPmPlanReq = new SearchPmPlanReq();
//			searchPmPlanReq.setBentelerPlanId(pmPlan.getBentelerPlanId());
//			List<PmTaskListRes> listRes = this.getTaskList(searchPmPlanReq);
//			int planStatus = 0; // 计划今天的状态，0未完成 1已完成 2待重检
//			int count0 = 0; // 任务的task.getStatus=0初始化
//			int count1 = 0; // 任务的task.getStatus=1待审核状态
//            int count2 = 0; // 任务的task.getStatus=2放行
//            int count3 = 0; // 任务的task.getStatus=3不放行
//			for (PmTaskListRes task : listRes) {
//				if (task.getStatus() == 0) {
//					count0++;
//				} else if (task.getStatus() == 1) {
//					count1++;
//				} else if (task.getStatus() == 2) {
//					count2++;
//				} else if (task.getStatus() == 3) {
//					count3++;
//				}
//			}
//			if (count0 > 0 || count1 > 0) {
//				planStatus = 0; // 存在未完成的任务，计划状态0未完成
//			} else if (count2 == listRes.size()) {
//				planStatus = 1; // 所有任务都检查完毕，都放行，计划状态1已完成
//			} else if (count3 > 0) {
//				planStatus = 2; // 所有任务都检查完毕，存在不放行记录，计划状态2待重检
//			}
//
//			pmPlan.setPlanStatus(planStatus);
//		}

		return ResponseBeanBuilder.ok(listResPage, Function.identity());
	}

	/**
     * 取得防错计划当天任务信息
     * @param req
     * @return 防错计划当天任务信息
     */
	@Override
    public List<PmTaskListRes> getTaskList(SearchPmPlanReq req) {
		List<PmTaskListRes> listRes = preventMistakeMapper.getTaskList(req);
		return listRes;
	}

	/**
     * 取得防错计划信息
     * @param req
     * @return 防错计划信息
     */
	@Override
    public PmPlanRes getPmPlan(SearchPmPlanReq req) {
		BentelerPlan plan = planService.getById(req.getBentelerPlanId());
		PmPlanRes pmPlanRes = new PmPlanRes();
		pmPlanRes.setBentelerPlanId(plan.getId());
		pmPlanRes.setBentelerPlanName(plan.getName());
		pmPlanRes.setPlanType(plan.getPlanType());
		pmPlanRes.setEffectiveDate(plan.getEffectiveDate());;
		pmPlanRes.setCreatedUserId(plan.getCreatedUserId());
		List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarService.list(Wrappers.<OnsiteCalendar>lambdaQuery()
                .eq(OnsiteCalendar::getBentelerPlanId, plan.getId()));
		pmPlanRes.setOnsiteCalendarList(onsiteCalendarList);

		try {
			OnsiteTitleBlob onsiteTitleBlob = onsiteTitleBlobService.getOne(new LambdaQueryWrapper<OnsiteTitleBlob>()
	    			.eq(OnsiteTitleBlob::getBentelerPlanId, req.getBentelerPlanId()));
			pmPlanRes.setOnsiteTitle((OnsiteTitle)this.byteArrayToObject(onsiteTitleBlob.getTitleContent()));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                .eq(OnsiteWorkshop::getBentelerPlanId, plan.getId()));
		for (OnsiteWorkshop onsiteWorkshop : onsiteWorkshopList) {
			List<OnsiteItem> onsiteItemList = onsiteItemService.list(Wrappers.<OnsiteItem>lambdaQuery()
	                .eq(OnsiteItem::getWorkshopId, onsiteWorkshop.getId()));
			onsiteWorkshop.setOnsiteItemList(onsiteItemList);
		}
		pmPlanRes.setOnsiteWorkshopList(onsiteWorkshopList);

		return pmPlanRes;
    }

	/**
     * 禁用解禁防错计划
     * @param req
     * @return
     */
	@Override
    public ResponseBean disablePmPlan(PmPlanPauseFlagReq req) {
		planService.updateById(BentelerPlan.builder()
                .id(req.getBentelerPlanId())
                .pauseFlag(req.getPauseFlag())
                .build());
        return ResponseBeanBuilder.ok();
    }

	/**
     * 取得工作站信息
     * @param req
     * @return 工作站信息
     */
	@Override
    public ResponseBean<OnsiteWorkshop> getWorkshop(SearchWorkshopReq req) {

		OnsiteWorkshop onsiteWorkshop = onsiteWorkshopService.getById(req.getWorkshopId());

		List<OnsiteItem> onsiteItemList = onsiteItemService.list(Wrappers.<OnsiteItem>lambdaQuery()
                .eq(OnsiteItem::getWorkshopId, onsiteWorkshop.getId()));
		onsiteWorkshop.setOnsiteItemList(onsiteItemList);

		return ResponseBeanBuilder.ok(onsiteWorkshop);
    }

    /**
     * 修改防错计划
     * @param req
     * @return
     */
	@Override
	public ResponseBean editPmPlan(EditPmPlanReq req) {

		if (firstArticleService.checkApprovalPlan(req.getBentelerPlanId())) { // 审批通过
//			/* 校验计划名称是否存在 */
//			if (this.checkPlanNameExist(req.getBentelerPlanName())) {
//				return ResponseBeanBuilder.fail("计划名称已存在");
//			}

			/* 添加防错计划 */
			BentelerPlan plan = BentelerPlan.builder()
					.planType(PlanType.防错.value)
					.name(req.getBentelerPlanName())
					.pauseFlag(false)
					.effectiveDate(req.getEffectiveDate())
					.build();
			planService.save(plan);

			/* 添加日周月点检规则 */
			List<OnsiteCalendarReq> onsiteCalendarList = req.getOnsiteCalendarList();
			for (OnsiteCalendarReq onsiteCalendarReq : onsiteCalendarList) {
				OnsiteCalendar onsiteCalendar = OnsiteCalendar.builder()
						.bentelerPlanId(plan.getId())
						.startDay(onsiteCalendarReq.getStartDay())
						.endDay(onsiteCalendarReq.getEndDay())
						.dayValidate(onsiteCalendarReq.getDayValidate())
						.dayOuttime(onsiteCalendarReq.getDayOuttime())
						.weekDay(onsiteCalendarReq.getWeekDay())
						.monthDay(onsiteCalendarReq.getMonthDay())
						.month(onsiteCalendarReq.getMonth())
						.userId(onsiteCalendarReq.getUserId())
						.typeFlag(onsiteCalendarReq.getTypeFlag())
						.shift(onsiteCalendarReq.getShift())
						.build();
				onsiteCalendarService.save(onsiteCalendar);
			}

			/* 添加表头信息 */
			try {
				OnsiteTitleBlob onsiteTitleBlob = OnsiteTitleBlob.builder()
						.bentelerPlanId(plan.getId())
						.titleContent(this.objectToByteArray(req.getOnsiteTitle()))
						.build();
				onsiteTitleBlobService.save(onsiteTitleBlob);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			/* 添加工作站信息 */
			List<OnsiteWorkshopReq> onsiteWorkshopList = req.getOnsiteWorkshopList();
			for (OnsiteWorkshopReq onsiteWorkshopReq : onsiteWorkshopList) {
//				if (this.checkWsNameExist(onsiteWorkshopReq.getWsName())) {
//					return ResponseBeanBuilder.fail("工作站名称" + onsiteWorkshopReq.getWsName() + "已存在");
//				}

				OnsiteWorkshop onsiteWorkshop = OnsiteWorkshop.builder()
						.bentelerPlanId(plan.getId())
						.wsName(onsiteWorkshopReq.getWsName())
						.build();
				onsiteWorkshopService.save(onsiteWorkshop);

				/* 添加巡检项目信息 */
				List<OnsiteItemReq> onsiteItemList = onsiteWorkshopReq.getOnsiteItemList();
				for (OnsiteItemReq onsiteItemReq : onsiteItemList) {
					OnsiteItem onsiteItem = OnsiteItem.builder()
							.workshopId(onsiteWorkshop.getId())
							.itemContent(onsiteItemReq.getItemContent())
							.typeFlag(onsiteItemReq.getTypeFlag())
							.inspectionFlag(onsiteItemReq.getInspectionFlag())
							.build();
					onsiteItemService.save(onsiteItem);
				}
			}

			/* 取得审批人 approvalLevel=1级审批 type=1计划审批 */
			ApprovalUserRes approvalUserRes = this.getApprovalUserInfo(1, 1);
			if (approvalUserRes != null) {
				/* 添加计划一级审批 */
				OnsiteApprovalPlan onsiteApprovalPlan = OnsiteApprovalPlan.builder()
						.bentelerPlanId(plan.getId())
						.approvalLevel(1) // 1级审批
						.approvalUserId(approvalUserRes.getUserId())
						.status(0) // 0未审批
						.build();
				onsiteApprovalPlanService.save(onsiteApprovalPlan);
			}

			/* 绑定计划 */
			PlanBind planBind = PlanBind.builder()
					.oldBentelerPlanId(req.getBentelerPlanId())
					.newBentelerPlanId(plan.getId())
					.build();
			planBindService.save(planBind);

		} else { // 没审批通过
			BentelerPlan oldPlan = planService.getById(req.getBentelerPlanId());
			if (!req.getBentelerPlanName().equals(oldPlan.getName())) {
				/* 校验计划名称是否存在 */
				if (this.checkPlanNameExist(req.getBentelerPlanName())) {
					return ResponseBeanBuilder.fail("计划名称已存在");
				}
			}

//			/* 将之前的plan所有信息记到his中（暂时没记审批信息） */
//			int count = onsiteApprovalPlanService.count(
//					Wrappers.<OnsiteApprovalPlan> lambdaQuery()
//					.eq(OnsiteApprovalPlan::getBentelerPlanId, req.getBentelerPlanId())
//					.eq(OnsiteApprovalPlan::getStatus, 2));
//			if (count > 0) { // 当前审批状态为驳回
//				SearchPmPlanReq searchPmPlanReq = new SearchPmPlanReq();
//				searchPmPlanReq.setBentelerPlanId(req.getBentelerPlanId());
//				PmPlanRes pmPlanRes = this.getPmPlan(searchPmPlanReq);
//				try {
//					OnsitePlanHisBlob onsitePlanHisBlob = OnsitePlanHisBlob.builder()
//							.bentelerPlanId(req.getBentelerPlanId())
//							.hisContent(this.objectToByteArray(pmPlanRes))
//							.build();
//					onsitePlanHisBlobService.save(onsitePlanHisBlob);
//				} catch (Exception e) {
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//				}
//			}

			/* 修改防错计划 */
			BentelerPlan plan = BentelerPlan.builder()
					.id(req.getBentelerPlanId())
					.name(req.getBentelerPlanName())
					.effectiveDate(req.getEffectiveDate())
					.build();
			planService.updateById(plan);

			/* 修改日周月点检规则 */
			/* 删除日周月点检规则 */
			onsiteCalendarService.remove(Wrappers.<OnsiteCalendar>lambdaQuery()
	                .eq(OnsiteCalendar::getBentelerPlanId, plan.getId()));
			/* 添加日周月点检规则 */
			List<OnsiteCalendarReq> onsiteCalendarList = req.getOnsiteCalendarList();
			for (OnsiteCalendarReq onsiteCalendarReq : onsiteCalendarList) {
				OnsiteCalendar onsiteCalendar = OnsiteCalendar.builder()
						.bentelerPlanId(plan.getId())
						.startDay(onsiteCalendarReq.getStartDay())
						.endDay(onsiteCalendarReq.getEndDay())
						.dayValidate(onsiteCalendarReq.getDayValidate())
						.dayOuttime(onsiteCalendarReq.getDayOuttime())
						.weekDay(onsiteCalendarReq.getWeekDay())
						.monthDay(onsiteCalendarReq.getMonthDay())
						.month(onsiteCalendarReq.getMonth())
						.userId(onsiteCalendarReq.getUserId())
						.typeFlag(onsiteCalendarReq.getTypeFlag())
						.shift(onsiteCalendarReq.getShift())
						.build();
				onsiteCalendarService.save(onsiteCalendar);
			}

			/* 修改表头信息 */
			try {
				LambdaUpdateWrapper<OnsiteTitleBlob> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
		        lambdaUpdateWrapper.eq(OnsiteTitleBlob::getBentelerPlanId, plan.getId())
		                           .set(OnsiteTitleBlob::getTitleContent, this.objectToByteArray(req.getOnsiteTitle()));
		        onsiteTitleBlobService.update(lambdaUpdateWrapper);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

	        /* 修改工作站、巡检项目 */
	        /* 删除巡检项目 */
	        List<OnsiteWorkshop> onsiteWorkshopListTemp = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
	                .eq(OnsiteWorkshop::getBentelerPlanId, plan.getId()));
	        List<Long> workshopIdList = new ArrayList<Long>();
	        for (OnsiteWorkshop onsiteWorkshop : onsiteWorkshopListTemp) {
	        	workshopIdList.add(onsiteWorkshop.getId());
	        }
	        onsiteItemService.remove(Wrappers.<OnsiteItem>lambdaQuery()
	                .in(OnsiteItem::getWorkshopId, workshopIdList));
	        /* 删除工作站 */
	        onsiteWorkshopService.remove(Wrappers.<OnsiteWorkshop>lambdaQuery()
	                .eq(OnsiteWorkshop::getBentelerPlanId, plan.getId()));
	        /* 添加工作站信息 */
			List<OnsiteWorkshopReq> onsiteWorkshopList = req.getOnsiteWorkshopList();
			for (OnsiteWorkshopReq onsiteWorkshopReq : onsiteWorkshopList) {
//				if (this.checkWsNameExist(onsiteWorkshopReq.getWsName())) {
//					return ResponseBeanBuilder.fail("工作站名称" + onsiteWorkshopReq.getWsName() + "已存在");
//				}

				OnsiteWorkshop onsiteWorkshop = OnsiteWorkshop.builder()
						.bentelerPlanId(plan.getId())
						.wsName(onsiteWorkshopReq.getWsName())
						.build();
				onsiteWorkshopService.save(onsiteWorkshop);

				/* 添加巡检项目信息 */
				List<OnsiteItemReq> onsiteItemList = onsiteWorkshopReq.getOnsiteItemList();
				for (OnsiteItemReq onsiteItemReq : onsiteItemList) {
					OnsiteItem onsiteItem = OnsiteItem.builder()
							.workshopId(onsiteWorkshop.getId())
							.itemContent(onsiteItemReq.getItemContent())
							.typeFlag(onsiteItemReq.getTypeFlag())
							.inspectionFlag(onsiteItemReq.getInspectionFlag())
							.build();
					onsiteItemService.save(onsiteItem);
				}
			}

			/* 删除之前的审批记录 */
			onsiteApprovalPlanService.remove(Wrappers.<OnsiteApprovalPlan>lambdaQuery()
	                .eq(OnsiteApprovalPlan::getBentelerPlanId, plan.getId()));
			/* 取得审批人 approvalLevel=1级审批 type=1计划审批 */
			ApprovalUserRes approvalUserRes = this.getApprovalUserInfo(1, 1);
			if (approvalUserRes != null) {
				/* 添加计划一级审批 */
				OnsiteApprovalPlan onsiteApprovalPlan = OnsiteApprovalPlan.builder()
						.bentelerPlanId(plan.getId())
						.approvalLevel(1) // 1级审批
						.approvalUserId(approvalUserRes.getUserId())
						.status(0) // 0未审批
						.build();
				onsiteApprovalPlanService.save(onsiteApprovalPlan);
			}
		}

		return ResponseBeanBuilder.ok();
	}

	/**
     * 取得计划审批信息
     * @param req
     * @return 计划审批信息
     */
	@Override
    public ResponseBean<List<OnsiteApprovalPlan>> getPmPlanApprovalList(SearchPmPlanReq req) {
    	List<OnsiteApprovalPlan> onsiteApprovalPlanList = onsiteApprovalPlanService.list(Wrappers.<OnsiteApprovalPlan>lambdaQuery()
                .eq(OnsiteApprovalPlan::getBentelerPlanId, req.getBentelerPlanId()));
		return ResponseBeanBuilder.ok(onsiteApprovalPlanList, Function.identity());
    }

    /**
     * 取得巡检规则信息
     * @param req
     * @return 巡检规则信息
     */
    @Override
    public ResponseBean<List<OnsiteCalendar>> getCalendarList(SearchPmPlanReq req) {
    	List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarService.list(Wrappers.<OnsiteCalendar>lambdaQuery()
                .eq(OnsiteCalendar::getBentelerPlanId, req.getBentelerPlanId()));
		return ResponseBeanBuilder.ok(onsiteCalendarList, Function.identity());
    }

    /**
     * 取得检查记录列表
     * @param req
     * @return 检查记录列表
     */
    @Override
    public ResponseBean<List<PmCheckRecordListRes>> getPmCheckRecordList(SearchPmCheckRecordListReq req) {
    	List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                .eq(OnsiteWorkshop::getBentelerPlanId, req.getBentelerPlanId()));

    	List<PmCheckRecordListRes> workshopList = new ArrayList<PmCheckRecordListRes>();
    	PmCheckRecordListRes workshop = null;
    	for (OnsiteWorkshop onsiteWorkshop : onsiteWorkshopList) {
    		workshop = new PmCheckRecordListRes();
    		workshop.setWorkshopId(onsiteWorkshop.getId());
    		workshop.setWsName(onsiteWorkshop.getWsName());

			List<OnsiteItem> onsiteItemList = onsiteItemService.list(Wrappers.<OnsiteItem>lambdaQuery()
	                .eq(OnsiteItem::getWorkshopId, workshop.getWorkshopId()));

			List<PmCheckRecordItemListRes> itemList = new ArrayList<PmCheckRecordItemListRes>();
			PmCheckRecordItemListRes item = null;
			for (OnsiteItem onsiteItem : onsiteItemList) {
				item = new PmCheckRecordItemListRes();

				item.setOnsiteItemId(onsiteItem.getId());
				item.setItemContent(onsiteItem.getItemContent());

				SearchCheckRecordReq itemResultReq = new SearchCheckRecordReq();
	    		itemResultReq.setOnsiteItemId(item.getOnsiteItemId());
	    		itemResultReq.setYearMonth(req.getYearMonth());
	    		itemResultReq.setTypeFlag(req.getTypeFlag());
	    		List<PmCheckRecordRes> itemResultList = preventMistakeMapper.getPmItemResultList(itemResultReq);

	    		item.setCheckRecordList(itemResultList);

				itemList.add(item);
			}

			workshop.setItemList(itemList);

			workshopList.add(workshop);
		}
    	return ResponseBeanBuilder.ok(workshopList, Function.identity());

//    	SearchPmPlanReq workshopItemReq = new SearchPmPlanReq();
//    	workshopItemReq.setBentelerPlanId(req.getBentelerPlanId());
//    	List<PmCheckRecordListRes> workshopItemList = preventMistakeMapper.getPmWorkshopItemList(workshopItemReq);
//
//    	for (PmCheckRecordListRes workshopItem : workshopItemList) {
//    		SearchCheckRecordReq itemResultReq = new SearchCheckRecordReq();
//    		itemResultReq.setOnsiteItemId(workshopItem.getOnsiteItemId());
//    		itemResultReq.setYearMonth(req.getYearMonth());
//    		itemResultReq.setTypeFlag(req.getTypeFlag());
//    		List<PmCheckRecordRes> itemResultList = preventMistakeMapper.getPmItemResultList(itemResultReq);
//
//    		workshopItem.setCheckRecordList(itemResultList);
//    	}
//
//		return ResponseBeanBuilder.ok(workshopItemList, Function.identity());
    }

    /**
     * 取得Pad端防错计划列表
     * @param req
     * @return Pad端防错计划列表
     */
    @Override
    public ResponseBean<List<PmPlanListForPadRes>> getPmPlanListForPad(SearchPmPlanListForPadReq req) {
    	List<PmPlanListForPadRes> listRes = preventMistakeMapper.getPmPlanListForPad(req);
		return ResponseBeanBuilder.ok(listRes, Function.identity());
    }

    /**
     * 取得表头信息
     * @param bentelerPlanId
     * @return 表头信息
     */
    @Override
    public OnsiteTitle getTitle(Long bentelerPlanId) {
    	OnsiteTitleBlob onsiteTitleBlob = onsiteTitleBlobService.getOne(new LambdaQueryWrapper<OnsiteTitleBlob>()
    			.eq(OnsiteTitleBlob::getBentelerPlanId, bentelerPlanId));
    	OnsiteTitle onsiteTitle = new OnsiteTitle();
		try {
			onsiteTitle = (OnsiteTitle)this.byteArrayToObject(onsiteTitleBlob.getTitleContent());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return onsiteTitle;
    }

    /**
     * 取得工作站列表
     * @param req
     * @return 工作站列表
     */
    @Override
    public ResponseBean<List<OnsiteWorkshop>> getWorkshopList(SearchPmAllCheckFlagReq req) {
    	List<OnsiteWorkshop> onsiteWorkshopList = preventMistakeMapper.getPmWorkshopListForPad(req);
//    	List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
//                .eq(OnsiteWorkshop::getBentelerPlanId, req.getBentelerPlanId()));
    	SearchPmWsAllCheckFlagReq searchPmWsAllCheckFlagReq = null;
    	for (OnsiteWorkshop onsiteWorkshop : onsiteWorkshopList) {
    		/* 判断工作站下项目是否都检查完 */
    		searchPmWsAllCheckFlagReq = new SearchPmWsAllCheckFlagReq();
    		searchPmWsAllCheckFlagReq.setOnsiteTaskId(req.getOnsiteTaskId());
    		searchPmWsAllCheckFlagReq.setWorkshopId(onsiteWorkshop.getId());
    		searchPmWsAllCheckFlagReq.setTypeFlag(req.getTypeFlag());   		

    		int itemCount = preventMistakeMapper.getPmWsItemCount(searchPmWsAllCheckFlagReq);
    		int itemResultCount = preventMistakeMapper.getPmWsItemResultCount(searchPmWsAllCheckFlagReq);
    		if (itemCount == itemResultCount) {
    			onsiteWorkshop.setAllCheckFlag(1);
    		} else {
    			onsiteWorkshop.setAllCheckFlag(0);
    		}
    	}

		return ResponseBeanBuilder.ok(onsiteWorkshopList, Function.identity());
    }

    /**
     * 取得巡检项目列表
     * @param req
     * @return 巡检项目列表
     */
    @Override
    public ResponseBean<List<OnsiteItem>> getItemList(SearchItemListReq req) {
    	List<OnsiteItem> onsiteItemList = onsiteItemService.list(Wrappers.<OnsiteItem>lambdaQuery()
                .eq(OnsiteItem::getWorkshopId, req.getWorkshopId())
                .eq(OnsiteItem::getTypeFlag, req.getTypeFlag()));

    	for (OnsiteItem onsiteItem : onsiteItemList) {
    		OnsiteItemResult onsiteItemResult = onsiteItemResultService.getOne(new LambdaQueryWrapper<OnsiteItemResult>()
        			.eq(OnsiteItemResult::getOnsiteTaskId, req.getOnsiteTaskId())
        			.eq(OnsiteItemResult::getOnsiteItemId, onsiteItem.getId()));
    		if (onsiteItemResult != null) {
    			onsiteItem.setResult(onsiteItemResult.getResult());
    			onsiteItem.setAbnormal(onsiteItemResult.getAbnormal());
    		}
    	}

    	return ResponseBeanBuilder.ok(onsiteItemList, Function.identity());
    }

    /**
     * 添加巡检项目检查结果
     * @param req
     * @return
     */
    @Override
	public ResponseBean addItemResult(AddItemResultListReq req) {
        List<AddItemResultReq> addItemResultList = req.getAddItemResultList();
        
        /* 删除上一次保存结果 */
        List<Long> onsiteItemIdList = new ArrayList<Long>();
        for (AddItemResultReq itemResult : addItemResultList) {
        	onsiteItemIdList.add(itemResult.getOnsiteItemId());
        }
    	onsiteItemResultService.remove(Wrappers.<OnsiteItemResult>lambdaQuery()
                .eq(OnsiteItemResult::getOnsiteTaskId, req.getOnsiteTaskId())
                .in(OnsiteItemResult::getOnsiteItemId, onsiteItemIdList));

    	/* 添加新的检查结果 */
//    	List<AddItemResultReq> addItemResultList = req.getAddItemResultList();
    	for (AddItemResultReq addItemResult : addItemResultList) {
    		OnsiteItemResult onsiteItemResult = OnsiteItemResult.builder()
    				.onsiteTaskId(req.getOnsiteTaskId())
    				.onsiteItemId(addItemResult.getOnsiteItemId())
    				.result(addItemResult.getResult())
    				.abnormal(addItemResult.getAbnormal())
    				.build();
    		onsiteItemResultService.save(onsiteItemResult);
    	}

		return ResponseBeanBuilder.ok();
	}

	/**
     * 任务提交
     * @param req
     * @return
     */
	@Override
	public ResponseBean submitTask(SearchTaskReq req) {
		/* 取得task下的所有条目检查结果 */
		List<OnsiteItemResult> onsiteItemResultList = onsiteItemResultService.list(Wrappers.<OnsiteItemResult>lambdaQuery()
                .eq(OnsiteItemResult::getOnsiteTaskId, req.getOnsiteTaskId()));
		boolean flag = true; // true表示巡检项目都通过，false表示巡检项目有不通过的
		for (OnsiteItemResult onsiteItemResult : onsiteItemResultList) {
			if (onsiteItemResult.getResult() == 1) {
				flag = false;
				break;
			}
		}

		if (flag) { // 巡检项目都通过
			OnsiteTask onsiteTask = OnsiteTask.builder()
					.id(req.getOnsiteTaskId())
					.status(4)
					.build();
			onsiteTaskService.updateById(onsiteTask);
		} else { // 巡检项目有不通过的
			/* 更新任务为1待审核状态。等待后续审核 */
			OnsiteTask onsiteTask = OnsiteTask.builder()
					.id(req.getOnsiteTaskId())
					.status(1)
					.build();
			onsiteTaskService.updateById(onsiteTask);

			/* 删除之前的审批记录 */
			onsiteApprovalTaskService.remove(Wrappers.<OnsiteApprovalTask>lambdaQuery()
	                .eq(OnsiteApprovalTask::getOnsiteTaskId, onsiteTask.getId()));

			/* 取得审批人 approvalLevel=1级审批 type=3防错任务审批 */
			ApprovalUserRes approvalUserRes = this.getApprovalUserInfo(1, 3);
			if (approvalUserRes != null) {
				/* 添加任务一级审批 */
				OnsiteApprovalTask onsiteApprovalTask = OnsiteApprovalTask.builder()
						.onsiteTaskId(onsiteTask.getId())
						.approvalLevel(1) // 1级审批
						.approvalUserId(approvalUserRes.getUserId())
						.status(0) // 0初始化
						.build();
				onsiteApprovalTaskService.save(onsiteApprovalTask);
			}
		}

		return ResponseBeanBuilder.ok();
	}

	/**
     * 取得审批人
     * @param req
     * @return 审批人
     */
	@Override
    public ResponseBean<ApprovalUserRes> getApprovalUser(SearchApprovalUserReq req) {
    	ApprovalUserRes approvalUserRes = preventMistakeMapper.getApprovalUser(req);
		return ResponseBeanBuilder.ok(approvalUserRes);
    }

    /**
     * 取得审批人
     * @param approvalLevel 1是1级审批 2是2级审批
     * @param type 1计划审批  2首件任务审批  3防错任务审批
     * @return 审批人
     */
	@Override
    public ApprovalUserRes getApprovalUserInfo(int approvalLevel, int type) {
    	SearchApprovalUserReq searchApprovalUserReq = new SearchApprovalUserReq();
		searchApprovalUserReq.setApprovalLevel(approvalLevel); // 审批级别
		searchApprovalUserReq.setType(type); // 审批规则类型
		ApprovalUserRes approvalUserRes = preventMistakeMapper.getApprovalUser(searchApprovalUserReq);
        return approvalUserRes;
    }

    /**
	 * 取得每频次点检信息
	 *
	 * @param req
	 * @return 每频次点检信息
	 */
	@Override
	public ResponseBean<List<PmCheckListRes>> getPmCheckList(SearchPmPlanReq req) {
		List<PmCheckListRes> pmCheckList = preventMistakeMapper.getPmCheckList(req);
    	return ResponseBeanBuilder.ok(pmCheckList, Function.identity());
	}

	/**
	 * 取得不放行任务对应的工作站
	 *
	 * @param req
	 * @return 不放行任务对应的工作站
	 */
	@Override
	public ResponseBean<List<PmWorkshopNoGoListRes>> getPmWorkshopNoGoList(SearchPmWorkshopNoGoListReq req) {
		List<PmWorkshopNoGoListRes> pmWorkshopNoGoList = preventMistakeMapper.getPmWorkshopNoGoList(req);
    	return ResponseBeanBuilder.ok(pmWorkshopNoGoList, Function.identity());
	}

	/**
	 * 取得不放行任务对应的点检项目
	 *
	 * @param req
	 * @return 不放行任务对应的点检项目
	 */
	@Override
	public ResponseBean<List<PmItemNoGoListRes>> getPmItemNoGoList(SearchPmItemNoGoListReq req) {
		List<PmItemNoGoListRes> pmItemNoGoList = preventMistakeMapper.getPmItemNoGoList(req);
    	return ResponseBeanBuilder.ok(pmItemNoGoList, Function.identity());
	}

	/**
	 * 取得不放行任务对应的质量经理审批信息
	 *
	 * @param req
	 * @return 不放行任务对应的质量经理审批信息
	 */
	@Override
	public ResponseBean<PmNoGoTaskAapprovalRes> getPmNoGoTaskAapproval(SearchTaskReq req) {
		PmNoGoTaskAapprovalRes pmNoGoTaskAapprovalRes = preventMistakeMapper.getPmNoGoTaskAapproval(req);
    	return ResponseBeanBuilder.ok(pmNoGoTaskAapprovalRes);
	}

	/**
	 * 修改巡检项目检查结果(这里只有重检走这里)
	 *
	 * @param req
	 * @return
	 */
	@Override
	public ResponseBean editItemResult(EditItemResultListReq req) {
//		/* 任务和巡检结果记录历史 */
//		PmTaskRes pmTaskRes = new PmTaskRes();
//		OnsiteTask onsiteTask = onsiteTaskService.getById(req.getOnsiteTaskId());
//		pmTaskRes.setOnsiteTaskId(onsiteTask.getId());
//		pmTaskRes.setOnsiteCalendarId(onsiteTask.getOnsiteCalendarId());
//		pmTaskRes.setStatus(onsiteTask.getStatus());
//		pmTaskRes.setTaskTime(onsiteTask.getTaskTime());
//		pmTaskRes.setRemindTaskFlag(onsiteTask.getRemindTaskFlag());
//		pmTaskRes.setRemindNoworkFalg(onsiteTask.getRemindNoworkFalg());
//		List<OnsiteItemResult> onsiteItemResultList = onsiteItemResultService.list(Wrappers.<OnsiteItemResult>lambdaQuery()
//                .eq(OnsiteItemResult::getOnsiteTaskId, req.getOnsiteTaskId()));
//		pmTaskRes.setOnsiteItemResultList(onsiteItemResultList);
//
//		try {
//			OnsiteTaskHisBlob onsiteTaskHisBlob = OnsiteTaskHisBlob.builder()
//					.onsiteTaskId(req.getOnsiteTaskId())
//					.hisContent(this.objectToByteArray(pmTaskRes))
//					.build();
//			onsiteTaskHisBlobService.save(onsiteTaskHisBlob);
//		} catch (Exception e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}

		/* 更新巡检项目检查结果 */
		List<EditItemResultReq> editItemResultList = req.getEditItemResultList();
    	for (EditItemResultReq editItemResult : editItemResultList) {
    		onsiteItemResultService.remove(Wrappers.<OnsiteItemResult>lambdaQuery()
                    .eq(OnsiteItemResult::getOnsiteTaskId, req.getOnsiteTaskId())
                    .eq(OnsiteItemResult::getOnsiteItemId, editItemResult.getOnsiteItemId()));

    		OnsiteItemResult onsiteItemResult = OnsiteItemResult.builder()
    				.onsiteTaskId(req.getOnsiteTaskId())
    				.onsiteItemId(editItemResult.getOnsiteItemId())
    				.result(editItemResult.getResult())
    				.abnormal(editItemResult.getAbnormal())
    				.build();
    		onsiteItemResultService.save(onsiteItemResult);
    	}

		return ResponseBeanBuilder.ok();
	}

	/**
     * 修改防错计划
     * @param req
     * @return
     */
	@Override
	public void setOnsitePlanHis(Long bentelerPlanId) {

		SearchPmPlanReq searchPmPlanReq = new SearchPmPlanReq();
		searchPmPlanReq.setBentelerPlanId(bentelerPlanId);
		PmPlanRes pmPlanRes = this.getPmPlan(searchPmPlanReq);
		try {
			OnsitePlanHisBlob onsitePlanHisBlob = OnsitePlanHisBlob.builder()
					.bentelerPlanId(bentelerPlanId)
					.hisContent(this.objectToByteArray(pmPlanRes))
					.build();
			onsitePlanHisBlobService.save(onsitePlanHisBlob);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 任务和巡检结果记录历史
	 *
	 * @param onsiteTaskId
	 * @return
	 */
	@Override
	public void setOnsiteTaskHis(Long onsiteTaskId) {
		/* 任务和巡检结果记录历史 */
		PmTaskRes pmTaskRes = new PmTaskRes();
		OnsiteTask onsiteTask = onsiteTaskService.getById(onsiteTaskId);
		pmTaskRes.setOnsiteTaskId(onsiteTask.getId());
		pmTaskRes.setOnsiteCalendarId(onsiteTask.getOnsiteCalendarId());
		pmTaskRes.setStatus(onsiteTask.getStatus());
		pmTaskRes.setTaskTime(onsiteTask.getTaskTime());
		pmTaskRes.setRemindTaskFlag(onsiteTask.getRemindTaskFlag());
		pmTaskRes.setRemindNoworkFlag(onsiteTask.getRemindNoworkFlag());
		List<OnsiteItemResult> onsiteItemResultList = onsiteItemResultService.list(Wrappers.<OnsiteItemResult>lambdaQuery()
                .eq(OnsiteItemResult::getOnsiteTaskId, onsiteTaskId));
		pmTaskRes.setOnsiteItemResultList(onsiteItemResultList);

		try {
			OnsiteTaskHisBlob onsiteTaskHisBlob = OnsiteTaskHisBlob.builder()
					.onsiteTaskId(onsiteTaskId)
					.hisContent(this.objectToByteArray(pmTaskRes))
					.build();
			onsiteTaskHisBlobService.save(onsiteTaskHisBlob);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 取得所有工作站
	 *
	 * @param req
	 * @return 所有工作站
	 */
	@Override
	public ResponseBean<List<PmAllWorkshopListRes>> getAllWorkshopList(SearchAllWorkshopReq req) {
		QueryWrapper<OnsiteWorkshop> queryWrapper = new QueryWrapper<>();
		if (req.getWsName() != null && !"".equals(req.getWsName())) {
			queryWrapper.lambda().like(OnsiteWorkshop::getWsName, req.getWsName());
		}
		List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(queryWrapper);

		List<PmAllWorkshopListRes> pmAllWorkshopList = new ArrayList<PmAllWorkshopListRes>();
		PmAllWorkshopListRes pmAllWorkshop = null;
		for (OnsiteWorkshop onsiteWorkshop : onsiteWorkshopList) {
			pmAllWorkshop = new PmAllWorkshopListRes();
			pmAllWorkshop.setBentelerPlanId(onsiteWorkshop.getBentelerPlanId());
			pmAllWorkshop.setWorkshopId(onsiteWorkshop.getId());
			pmAllWorkshop.setWsName(onsiteWorkshop.getWsName());

			pmAllWorkshopList.add(pmAllWorkshop);
		}
		return ResponseBeanBuilder.ok(pmAllWorkshopList, Function.identity());
	}

	/**
     * 取得计划的最新审批信息
     * @param req
     * @return 计划的最新审批信息
     */
	@Override
    public OnsiteApprovalPlan getLatestApprovalInfo(SearchPmPlanReq req) {
		OnsiteApprovalPlan onsiteApprovalPlan = preventMistakeMapper.getLatestApprovalInfo(req);
        return onsiteApprovalPlan;
    }

    /**
     * 判断检验结果是否填写完成
     * @param req
     * @return 检验结果是否填写完成
     */
	@Override
    public PmAllCheckFlagRes getPmAllCheckFlag(SearchPmAllCheckFlagReq req) {
		SearchPmPlanReq searchPmPlanReq = new SearchPmPlanReq();
		searchPmPlanReq.setBentelerPlanId(req.getBentelerPlanId());
		int itemCount = preventMistakeMapper.getPmItemCount(searchPmPlanReq);

		int itemResultCount = preventMistakeMapper.getPmItemResultCount(req);

		PmAllCheckFlagRes pmAllCheckFlagRes = new PmAllCheckFlagRes();

		if (itemCount == itemResultCount) {
			pmAllCheckFlagRes.setAllCheckFlag(1); // 全部完成
		} else {
			pmAllCheckFlagRes.setAllCheckFlag(0); // 没完成
		}

		return pmAllCheckFlagRes;
	}

	/**
	 * 取得异常列表
	 *
	 * @param req
	 * @return 异常列表
	 */
	@Override
	public ResponseBean<List<AbnormalTaskInfoRes>> getAbnormalList(SearchAbnormalListReq req) {
		/* 防错 */
		SearchAbnormalReq searchReq = new SearchAbnormalReq();
		searchReq.setCheckDate(req.getCheckDate());
		List<AbnormalTaskInfoRes> pmReleaseList = preventMistakeMapper.getPmReleaseList(searchReq);
		List<AbnormalTaskInfoRes> pmRepeatList = preventMistakeMapper.getPmRepeatList(searchReq);
		List<AbnormalTaskInfoRes> pmList = new ArrayList<AbnormalTaskInfoRes>();
		for (AbnormalTaskInfoRes pmRelease : pmReleaseList) {
			pmList.add(pmRelease);
		}
		for (AbnormalTaskInfoRes pmRepeat : pmRepeatList) {
			pmList.add(pmRepeat);
		}
		for (AbnormalTaskInfoRes pmInfo : pmList) {
			List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
	                .eq(OnsiteWorkshop::getBentelerPlanId, pmInfo.getBentelerPlanId()));

			List<AbnormalWorkshopRes> abnormalWorkshopList = new ArrayList<AbnormalWorkshopRes>();
			AbnormalWorkshopRes abnormalWorkshop = null;
			for (OnsiteWorkshop workshop : onsiteWorkshopList) {
				abnormalWorkshop = new AbnormalWorkshopRes();
				abnormalWorkshop.setWorkshopId(workshop.getId());
				abnormalWorkshop.setWsName(workshop.getWsName());
				abnormalWorkshopList.add(abnormalWorkshop);
			}

			for (AbnormalWorkshopRes abnormalWorkshopInfo : abnormalWorkshopList) {
				searchReq = new SearchAbnormalReq();
				searchReq.setWorkshopId(abnormalWorkshopInfo.getWorkshopId());
				searchReq.setTaskId(pmInfo.getTaskId());
				List<AbnormalItemRes> abnormalItemList = preventMistakeMapper.getPmAbnormalItemList(searchReq);
				abnormalWorkshopInfo.setAbnormalItemList(abnormalItemList);
			}

			pmInfo.setAbnormalWorkshopList(abnormalWorkshopList);
		}

		/* 首件 */
		searchReq = new SearchAbnormalReq();
		searchReq.setCheckDate(req.getCheckDate());
		List<AbnormalTaskInfoRes> faReleaseList = preventMistakeMapper.getFaReleaseList(searchReq);
		List<AbnormalTaskInfoRes> faRepeatList = preventMistakeMapper.getFaRepeatList(searchReq);

		List<AbnormalTaskInfoRes> faList = new ArrayList<AbnormalTaskInfoRes>();
		for (AbnormalTaskInfoRes faRelease : faReleaseList) {
			faList.add(faRelease);
		}
		for (AbnormalTaskInfoRes faRepeat : faRepeatList) {
			faList.add(faRepeat);
		}

		for (AbnormalTaskInfoRes faInfo : faList) {
			List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
	                .eq(FirstArticleWorkshop::getBentelerPlanId, faInfo.getBentelerPlanId()));

			List<AbnormalWorkshopRes> abnormalWorkshopList = new ArrayList<AbnormalWorkshopRes>();
			AbnormalWorkshopRes abnormalWorkshop = null;
			for (FirstArticleWorkshop workshop : firstArticleWorkshopList) {
				abnormalWorkshop = new AbnormalWorkshopRes();
				abnormalWorkshop.setWorkshopId(workshop.getId());
				abnormalWorkshop.setWsName(workshop.getWsName());
				abnormalWorkshopList.add(abnormalWorkshop);
			}

			for (AbnormalWorkshopRes abnormalWorkshopInfo : abnormalWorkshopList) {
				searchReq = new SearchAbnormalReq();
				searchReq.setWorkshopId(abnormalWorkshopInfo.getWorkshopId());
				List<AbnormalItemRes> abnormalItemList = preventMistakeMapper.getFaAbnormalItemList(searchReq);

				for (AbnormalItemRes abnormalItemInfo : abnormalItemList) {
					List<AbnormalItemResultRes> firstItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 0);
					abnormalItemInfo.setFirstItemResultList(firstItemResultList);

					List<AbnormalItemResultRes> twoHundredItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 1);
					abnormalItemInfo.setTwoHundredItemResultList(twoHundredItemResultList);

					List<AbnormalItemResultRes> catchItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 2);
					abnormalItemInfo.setCatchItemResultList(catchItemResultList);
				}

				abnormalWorkshopInfo.setAbnormalItemList(abnormalItemList);
			}

			faInfo.setAbnormalWorkshopList(abnormalWorkshopList);
		}

		List<AbnormalTaskInfoRes> abnormalList = new ArrayList<AbnormalTaskInfoRes>();
		for (AbnormalTaskInfoRes pm : pmList) {
			abnormalList.add(pm);
		}
		for (AbnormalTaskInfoRes fa : faList) {
			abnormalList.add(fa);
		}

		return ResponseBeanBuilder.ok(abnormalList, Function.identity());
	}



	/**
	 * 取得检查结果
	 *
	 * @param taskId
	 * @param itemId
	 * @param typeFlag 0表示首件 1表示200件 2表示中断
	 * @return 检查结果
	 */
	private List<AbnormalItemResultRes> getAbnormalItemResultList(long taskId, long itemId, int typeFlag) {
		SearchAbnormalReq searchReq = new SearchAbnormalReq();
		searchReq.setTaskId(taskId);
		searchReq.setItemId(itemId);
		searchReq.setTypeFlag(typeFlag);
		List<AbnormalItemResultRes> abnormalItemResultList = preventMistakeMapper.getFaAbnormalItemResultList(searchReq);
	    return abnormalItemResultList;
	}


    /**
     * Object转成byte[]
     * @param obj
     * @return byte[]
     */
    private byte[] objectToByteArray(Object obj) throws Exception {
		byte[] bytes = null;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(obj);
		oos.close();
		bytes = baos.toByteArray();
		baos.close();
		return bytes;
	}

    /**
     * byte[]转成Object
     * @param byte[]
     * @return Object
     */
    private Object byteArrayToObject(byte[] buffer) throws Exception {
		Object ob = null;
		ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(buffer));
		ob = ois.readObject();
		ois.close();
		return ob;
	}


}
