package com.factory.wx.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.entity.pm.req.SearchAbnormalReq;
import com.factory.web.entity.pm.res.AbnormalItemRes;
import com.factory.web.entity.pm.res.AbnormalItemResultRes;
import com.factory.web.entity.pm.res.AbnormalTaskInfoRes;
import com.factory.web.entity.pm.res.AbnormalWorkshopRes;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.FileMapper;
import com.factory.web.mapper.pm.PreventMistakeMapper;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.fa.FirstArticleWorkshopService;
import com.factory.web.service.impl.FileEntry;
import com.factory.web.service.pm.OnsiteWorkshopService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.AbnormalDetailReq;
import com.factory.wx.entity.req.AbnormalPlanReq;
import com.factory.wx.entity.res.AbnormalDetailRes;
import com.factory.wx.entity.res.AbnormalPlanRes;
import com.factory.wx.mapper.AbnormalStatisticsMapper;
import com.factory.wx.service.AbnormalStatisticsService;
import com.factory.wx.service.WxPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-23 1:33 上午
 */
@Service("abnormalStatisticsService")
public class AbnormalStatisticsServiceImpl implements AbnormalStatisticsService {
    @Autowired
    private AbnormalStatisticsMapper abnormalStatisticsMapper;
    @Autowired
    private FileMapper fileMapper;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private PreventMistakeMapper preventMistakeMapper;
    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private OnsiteWorkshopService onsiteWorkshopService;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private WxPlanService wxPlanService;

    /**
     * 根据年月获取异常计划
     *
     * @param abnormalPlanReq
     * @return
     */
    @Override
    public List<AbnormalPlanRes> getExceptionPlansByMonth(AbnormalPlanReq abnormalPlanReq) {
        String month = getMonth(abnormalPlanReq);
        abnormalPlanReq.setMonth(month);

        List<AbnormalPlanRes> releasedAbnormalPlans = new ArrayList<>();
        //已放行
        if (abnormalPlanReq.getAbnormalType() == 2) {
            releasedAbnormalPlans.addAll(abnormalStatisticsMapper.getReleasedAbnormalPlans(abnormalPlanReq));
        }
        //已重检
        if (abnormalPlanReq.getAbnormalType() == 1) {
            releasedAbnormalPlans.addAll(abnormalStatisticsMapper.getRecheckedAbnormalPlans(abnormalPlanReq));
        }
        return releasedAbnormalPlans;
    }

    /**
     * 取得检查结果
     *
     * @param taskId
     * @param itemId
     * @param typeFlag 0表示首件 1表示200件 2表示中断
     * @return 检查结果
     */
    private List<AbnormalItemResultRes> getAbnormalItemResultList(long taskId, long itemId, int typeFlag) {
        SearchAbnormalReq searchReq = new SearchAbnormalReq();
        searchReq.setTaskId(taskId);
        searchReq.setItemId(itemId);
        searchReq.setTypeFlag(typeFlag);
        List<AbnormalItemResultRes> abnormalItemResultList = preventMistakeMapper.getFaAbnormalItemResultList(searchReq);
        return abnormalItemResultList;
    }

    /**
     * 异常放行详情
     *
     * @param abnormalDetailReq
     * @return
     */
    @Override
    public AbnormalDetailRes getAbnormalDetail(AbnormalDetailReq abnormalDetailReq) {
        AbnormalDetailRes abnormalDetailRes = new AbnormalDetailRes();
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(abnormalDetailReq.getPlanId());
        abnormalDetailRes.setPlanId(bentelerPlan.getId());
        abnormalDetailRes.setPlanName(bentelerPlan.getName());

        //获取title
        if (abnormalDetailReq.getCheckType() == 1) {
            FirstArticleTitle title = firstArticleService.getFaTitle(abnormalDetailReq.getPlanId());
            //判断是否有附件
            int attrCount = fileMapper.selectCount(Wrappers.<FileEntry>lambdaQuery()
                    .eq(FileEntry::getId, title.getUploadAttachmentId()));
            abnormalDetailRes.setFirstArticleTitle(title);
            abnormalDetailRes.setAttachment(attrCount == 0 ? 0 : 1);
        }
        if (abnormalDetailReq.getCheckType() == 2) {
            //获取title
            OnsiteTitle title = preventMistakeService.getTitle(abnormalDetailReq.getPlanId());
            //判断是否有附件
            int attrCount = fileMapper.selectCount(Wrappers.<FileEntry>lambdaQuery()
                    .eq(FileEntry::getId, title.getUploadAttachmentId()));
            abnormalDetailRes.setTitleInfo(title);
            abnormalDetailRes.setAttachment(attrCount == 0 ? 0 : 1);
        }

        //取放行信息
        if (abnormalDetailReq.getCheckType() == 1) {
            List<AbnormalTaskInfoRes> abnormalTaskInfoResList = getFaAbnormalTaskInfoRes(abnormalDetailReq.getPlanId());
            abnormalDetailRes.setAbnormalTaskInfos(abnormalTaskInfoResList);
        }
        if (abnormalDetailReq.getCheckType() == 2) {
            List<AbnormalTaskInfoRes> abnormalTaskInfoResList = getPMAbnormalTaskInfoRes(abnormalDetailReq.getPlanId());
            abnormalDetailRes.setAbnormalTaskInfos(abnormalTaskInfoResList);
        }

        return abnormalDetailRes;
    }

    /**
     * 获取首件放行信息
     *
     * @param planId
     * @return
     */
    private List<AbnormalTaskInfoRes> getFaAbnormalTaskInfoRes(Long planId) {
        List<AbnormalTaskInfoRes> faList = abnormalStatisticsMapper.getFaReleaseListByPlanId(planId);

        for (AbnormalTaskInfoRes faInfo : faList) {
            List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                    .eq(FirstArticleWorkshop::getBentelerPlanId, faInfo.getBentelerPlanId()));

            List<AbnormalWorkshopRes> abnormalWorkshopList = new ArrayList<AbnormalWorkshopRes>();
            AbnormalWorkshopRes abnormalWorkshop = null;
            for (FirstArticleWorkshop workshop : firstArticleWorkshopList) {
                abnormalWorkshop = new AbnormalWorkshopRes();
                abnormalWorkshop.setWorkshopId(workshop.getId());
                abnormalWorkshop.setWsName(workshop.getWsName());
                abnormalWorkshopList.add(abnormalWorkshop);
            }

            for (int i = 0; i < abnormalWorkshopList.size(); i++) {

                SearchAbnormalReq searchReq = new SearchAbnormalReq();
                searchReq.setWorkshopId(abnormalWorkshopList.get(i).getWorkshopId());
                List<AbnormalItemRes> abnormalItemList = preventMistakeMapper.getFaAbnormalItemList(searchReq);

                for (AbnormalItemRes abnormalItemInfo : abnormalItemList) {
                    List<AbnormalItemResultRes> firstItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 0);
                    abnormalItemInfo.setFirstItemResultList(firstItemResultList);

                    List<AbnormalItemResultRes> twoHundredItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 1);
                    abnormalItemInfo.setTwoHundredItemResultList(twoHundredItemResultList);

                    List<AbnormalItemResultRes> catchItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 2);
                    abnormalItemInfo.setCatchItemResultList(catchItemResultList);
                }

                abnormalWorkshopList.get(i).setAbnormalItemList(abnormalItemList);
                faInfo.setAbnormalWorkshopList(abnormalWorkshopList);
            }
        }
        return faList;
    }

    /**
     * 获取防错放行信息
     *
     * @param planId
     * @return
     */
    private List<AbnormalTaskInfoRes> getPMAbnormalTaskInfoRes(Long planId) {
        List<AbnormalTaskInfoRes> pmList = abnormalStatisticsMapper.getPmReleaseListByPlanId(planId);
        for (AbnormalTaskInfoRes pmInfo : pmList) {
            List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                    .eq(OnsiteWorkshop::getBentelerPlanId, pmInfo.getBentelerPlanId()));

            List<AbnormalWorkshopRes> abnormalWorkshopList = new ArrayList<AbnormalWorkshopRes>();
            for (OnsiteWorkshop workshop : onsiteWorkshopList) {
                AbnormalWorkshopRes abnormalWorkshop = new AbnormalWorkshopRes();
                abnormalWorkshop.setWorkshopId(workshop.getId());
                abnormalWorkshop.setWsName(workshop.getWsName());
                abnormalWorkshopList.add(abnormalWorkshop);
            }

            for (int i = 0; i < abnormalWorkshopList.size(); i++) {

                SearchAbnormalReq searchReq = new SearchAbnormalReq();
                searchReq.setWorkshopId(abnormalWorkshopList.get(i).getWorkshopId());
                searchReq.setTaskId(pmInfo.getTaskId());
                List<AbnormalItemRes> abnormalItemList = preventMistakeMapper.getPmAbnormalItemList(searchReq).stream().filter(abnormalItemRes -> abnormalItemRes.getResult() > 0).collect(Collectors.toList());
                for (AbnormalItemRes abnormalItemRes : abnormalItemList) {
                    Long userId = abnormalItemRes.getItemResultUserId();
                    if (null != userId) {
                        String userName = wxPlanService.getUserNameById(abnormalItemRes.getItemResultUserId());
                        abnormalItemRes.setItemResultUser(userName);
                    }
                }
                abnormalWorkshopList.get(i).setAbnormalItemList(abnormalItemList);
            }

            pmInfo.setAbnormalWorkshopList(abnormalWorkshopList.stream().filter(workshop -> workshop.getAbnormalItemList().size() > 0).collect(Collectors.toList()));
        }
        return pmList;
    }

    /**
     * 验证日期格式
     *
     * @param abnormalPlanReq 默认2021-06
     * @return
     */
    private String getMonth(AbnormalPlanReq abnormalPlanReq) {
        String month = "2021-06";
        try {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM");
            Date date = simpleDateFormat.parse(abnormalPlanReq.getMonth());
            month = simpleDateFormat.format(date);
        } catch (ParseException e) {
            e.printStackTrace();
            ResponseBeanBuilder.fail("年月格式错误");
        }
        return month;
    }

}
