package com.factory.wx.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.fa.FirstArticleItem;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.fa.FirstArticleItemMapper;
import com.factory.web.mapper.fa.FirstArticleWorkshopMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalPlanMapper;
import com.factory.web.mapper.pm.OnsiteItemMapper;
import com.factory.web.mapper.pm.OnsiteWorkshopMapper;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalReq;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanApprovalDetailRes;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.service.PlanApprovalService;
import com.factory.wx.service.WxPlanService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Service("planApprovalService")
public class PlanApprovalServiceImpl implements PlanApprovalService {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanApprovalServiceImpl.class);
    /**
     * 审批规则类型为：1新建计划审批
     */
    private static int APPROVAL_TYPE_NEW = 1;
    /**
     * 审批规则类型为：3防错
     */
    private static int APPROVAL_TYPE_PM = 3;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteWorkshopMapper onsiteWorkshopMapper;
    @Autowired
    private OnsiteItemMapper onsiteItemMapper;
    @Autowired
    private OnsiteApprovalPlanMapper onsiteApprovalPlanMapper;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private FirstArticleWorkshopMapper firstArticleWorkshopMapper;
    @Autowired
    private FirstArticleItemMapper firstArticleItemMapper;
    @Autowired
    private WxPlanService wxPlanService;

    /**
     * 分页显示计划列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public PageBean<BentelerPlan> getApprovalListByPage(PlanPageReq planPageReq) {

        LambdaQueryWrapper<OnsiteApprovalPlan> approvalPlanWrapper = new LambdaQueryWrapper<>();
        approvalPlanWrapper.eq(OnsiteApprovalPlan::getApprovalUserId, planPageReq.getUserId());
        approvalPlanWrapper.eq(OnsiteApprovalPlan::getStatus, 0);//获取未处理过的计划
        List<OnsiteApprovalPlan> onsiteApprovalPlanList = onsiteApprovalPlanMapper.selectList(approvalPlanWrapper);

        IPage<BentelerPlan> bentelerPlanIPage = new Page<>();
        if (CollUtil.isNotEmpty(onsiteApprovalPlanList)) {
            LambdaQueryWrapper<BentelerPlan> wrapper = new LambdaQueryWrapper<>();
            List<Long> planIds = onsiteApprovalPlanList.stream().map(OnsiteApprovalPlan::getBentelerPlanId).collect(Collectors.toList());
            wrapper.in(BentelerPlan::getId, planIds);

            if (StringUtils.isNotEmpty(planPageReq.getName())) {
                wrapper.like(BentelerPlan::getName, planPageReq.getName());
            }
            wrapper.eq(BentelerPlan::getDeleted, 0);//未删除
            wrapper.eq(BentelerPlan::getPauseFlag, 0);//未禁用
            IPage<BentelerPlan> page = new Page<>(planPageReq.getPage(), planPageReq.getSize());
            bentelerPlanIPage = bentelerPlanMapper.selectPage(page, wrapper);
        }

        return PageBean.<BentelerPlan>builder().row(bentelerPlanIPage.getRecords()).total(bentelerPlanIPage.getTotal()).page(bentelerPlanIPage.getPages()).build();
    }

    /**
     * 获取审批计划详情-首件
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanApprovalDetailRes getApprovalFADetail(PlanInfoReq planInfoReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes =
                wxPlanService.getFirstArticlePlanBaseInfo(planInfoReq.getPlanId());

        List<FirstArticleWorkshop> firstArticleWorkshopList =
                firstArticleWorkshopMapper.selectList(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                        .eq(FirstArticleWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        firstArticleWorkshopList.forEach(firstArticleWorkshop -> {
            firstArticleWorkshop.setFirstArticleItemList(firstArticleItemMapper.selectList(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, firstArticleWorkshop.getId())));
        });

        List<Integer> itemType = getFAItemFrequency(firstArticleWorkshopList);

        return PlanApprovalDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .firstArticleTitle((FirstArticleTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .firstArticleWorkshops(firstArticleWorkshopList)
                .build();
    }

    /**
     * 首件频次归类
     *
     * @param firstArticleWorkshopList
     * @return
     */
    private List<Integer> getFAItemFrequency(List<FirstArticleWorkshop> firstArticleWorkshopList) {
        List<Integer> itemType = new ArrayList<>();
        firstArticleWorkshopList.forEach(workshop -> {
            workshop.getFirstArticleItemList().forEach(item -> {
                if (item.getFirstFlag() == 1) itemType.add(0);
                if (item.getTwoHundredFlag() == 1) itemType.add(1);
                if (item.getCatchFlag() == 1) itemType.add(2);
            });
        });
        return itemType.stream().distinct().collect(Collectors.toList());
    }


    /**
     * 获取审批计划详情-防错
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanApprovalDetailRes getApprovalPMDetail(PlanInfoReq planInfoReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes =
                wxPlanService.getOnsitePlanBaseInfo(planInfoReq.getPlanId());

        List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopMapper.selectList(Wrappers.<OnsiteWorkshop>lambdaQuery()
                .eq(OnsiteWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        onsiteWorkshopList.forEach(onsiteWorkshop -> {
            onsiteWorkshop.setOnsiteItemList(onsiteItemMapper.selectList(Wrappers.<OnsiteItem>lambdaQuery()
                    .eq(OnsiteItem::getWorkshopId, onsiteWorkshop.getId())));
        });
        List<Integer> itemType = getPMItemFrequency(onsiteWorkshopList);

        return PlanApprovalDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .onsiteWorkshops(onsiteWorkshopList)
                .build();
    }

    /**
     * 获取防错频次
     *
     * @param onsiteWorkshopList
     */
    private List<Integer> getPMItemFrequency(List<OnsiteWorkshop> onsiteWorkshopList) {
        List<Integer> frequency = new ArrayList<>();
        onsiteWorkshopList.forEach(workshop -> {
            workshop.getOnsiteItemList().forEach(item -> {
                frequency.add(item.getTypeFlag());
            });
        });
        return frequency.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 计划审批
     *
     * @param planApprovalReq
     * @return
     */
    @Override
    public ResponseBean approvalOperate(PlanApprovalReq planApprovalReq) {

        OnsiteApprovalPlan onsiteApprovalPlan = new OnsiteApprovalPlan();
        onsiteApprovalPlan.setStatus(planApprovalReq.getStatus());
        int updateStatus = onsiteApprovalPlanMapper.update(onsiteApprovalPlan, Wrappers.<OnsiteApprovalPlan>lambdaQuery()
                .eq(OnsiteApprovalPlan::getBentelerPlanId, planApprovalReq.getPlanId())
                .eq(OnsiteApprovalPlan::getApprovalUserId, planApprovalReq.getUserId()));
        if (updateStatus > 0) {
            //审批通过触发下一级别
            if (planApprovalReq.getStatus() == 1) {
                ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                        .eq(ApprovalRule::getType, APPROVAL_TYPE_NEW)
                        .orderByDesc(ApprovalRule::getApprovalLevel)
                        .last("limit 1"));
                //获取当前审批级别
                OnsiteApprovalPlan userApprovalInfo = onsiteApprovalPlanMapper.selectOne(Wrappers.<OnsiteApprovalPlan>lambdaQuery()
                        .eq(OnsiteApprovalPlan::getBentelerPlanId, planApprovalReq.getPlanId())
                        .eq(OnsiteApprovalPlan::getApprovalUserId, planApprovalReq.getUserId()).last("limit 1"));
                int userLevel = userApprovalInfo.getApprovalLevel();
                if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
                    ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_NEW);
                    onsiteApprovalPlanMapper.insert(OnsiteApprovalPlan.builder()
                            .bentelerPlanId(planApprovalReq.getPlanId())
                            .approvalLevel(approvalUserRes.getApprovalLevel())
                            .approvalUserId(approvalUserRes.getUserId())
                            .status(0)
                            .build());
                }
            }
            //审批不通过
            if (planApprovalReq.getStatus() == 2) {
                //审批不通过，保存审批历史
                preventMistakeService.setOnsitePlanHis(planApprovalReq.getPlanId());
            }
        }
        return ResponseBeanBuilder.ok();
    }

}
