package com.factory.wx.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalTaskMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.pm.OnsiteWorkshopService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanPassReq;
import com.factory.wx.entity.req.SearchPlanPassReq;
import com.factory.wx.entity.res.*;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.PlanPassPMService;
import com.factory.wx.service.WxPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-08-03 10:29 上午
 */

@Service("planPassPMService")
public class PlanPassPMServiceImpl implements PlanPassPMService {
    /**
     * 审批规则类型为：3防错
     */
    private static int APPROVAL_TYPE_PM = 3;
    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private OnsiteApprovalTaskMapper onsiteApprovalTaskMapper;
    @Autowired
    private WxPlanService wxPlanService;
    @Autowired
    private OnsiteWorkshopService onsiteWorkshopService;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;

    /**
     * 计划放行详情-防错
     *
     * @param planPassReq
     * @return
     */

    @Override
    public PlanPassDetailRes getPassDetailPM(PlanPassReq planPassReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getOnsitePlanBaseInfo(planPassReq.getPlanId());

        List<PlanPassTaskRes> planPassTaskResList = getPMTaskList(planPassReq);

        List<Integer> itemType = getItemPMFrequency(planPassTaskResList);

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .taskList(planPassTaskResList)
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .build();
    }

    /**
     * 获取防错点检错误列表
     *
     * @param planPassReq
     * @return
     */
    private List<PlanPassTaskRes> getPMTaskList(PlanPassReq planPassReq) {
        List<PlanPassTaskRes> pmList = planPassMapper.getPmPassTaskListByPlanId(planPassReq);
        for (PlanPassTaskRes pmInfo : pmList) {
            List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                    .eq(OnsiteWorkshop::getBentelerPlanId, pmInfo.getPlanId()));

            List<PlanPassWorkshopRes> workshopList = new ArrayList<PlanPassWorkshopRes>();
            for (OnsiteWorkshop workshop : onsiteWorkshopList) {
                PlanPassWorkshopRes planPassWorkshopRes = new PlanPassWorkshopRes();
                planPassWorkshopRes.setWorkshopId(workshop.getId());
                planPassWorkshopRes.setWsName(workshop.getWsName());
                workshopList.add(planPassWorkshopRes);
            }

            for (int i = 0; i < workshopList.size(); i++) {
                SearchPlanPassReq searchReq = new SearchPlanPassReq();
                searchReq.setWorkshopId(workshopList.get(i).getWorkshopId());
                searchReq.setTaskId(pmInfo.getTaskId());
                List<PlanPassItemRes> planPassItemList = planPassMapper.getPmPassItemList(searchReq).stream().filter(item -> item.getResult() > 0).collect(Collectors.toList());
                for (PlanPassItemRes planPassItemRes : planPassItemList) {
                    Long userId = planPassItemRes.getItemResultUserId();
                    if (null != userId) {
                        String userName = wxPlanService.getUserNameById(planPassItemRes.getItemResultUserId());
                        planPassItemRes.setItemResultUser(userName);
                    }
                }
                workshopList.get(i).setItemList(planPassItemList);
            }
            pmInfo.setWorkshopList(workshopList.stream().filter(workshop -> workshop.getItemList().size() > 0).collect(Collectors.toList()));
        }
        return pmList;
    }

    /**
     * 获取防错频次
     *
     * @param planPassTaskResList
     */
    private List<Integer> getItemPMFrequency(List<PlanPassTaskRes> planPassTaskResList) {
        List<Integer> frequency = new ArrayList<>();
        planPassTaskResList.forEach(task -> {
            task.getWorkshopList().forEach(workshop -> {
                workshop.getItemList().forEach(item -> {
                    frequency.add(item.getItemType());
                });
            });
        });
        return frequency.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean approvalPMTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {
        int updateStatus = 0; // 更新状态
        //更新防错审批信息
        updateStatus = updateOnsiteApprovalTask(planApprovalTaskReq);
        if (updateStatus > 0) {
            settingApprovalTask(planApprovalTaskReq);
        }
        return ResponseBeanBuilder.ok();
    }


    /**
     * 更新防错任务审批状态
     *
     * @param planApprovalTaskReq
     * @return
     */
    private int updateOnsiteApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        int updateStatus;
        OnsiteApprovalTask ot = new OnsiteApprovalTask();
        ot.setOnsiteTaskId(planApprovalTaskReq.getTaskId());
        ot.setReason(planApprovalTaskReq.getReason());
        ot.setStatus(planApprovalTaskReq.getStatus());
        updateStatus = onsiteApprovalTaskMapper.update(ot, Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        return updateStatus;
    }

    /**
     * 更新task状态
     *
     * @return
     */
    private int updateOnsiteTask(PlanApprovalTaskReq planApprovalTaskReq) {
        int taskStatus = 0; //task表状态
        if (planApprovalTaskReq.getStatus() == 1) taskStatus = 2; // 放行
        if (planApprovalTaskReq.getStatus() == 2) taskStatus = 3; // 不通过
        return onsiteTaskMapper.updateById(OnsiteTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
    }

    /**
     * 设置审批task任务
     *
     * @param planApprovalTaskReq
     * @return boolean 是否有下一级别
     */
    private void settingApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        //审批不通过触发下一级别
        if (planApprovalTaskReq.getStatus() == 2) {
            //设置防错下一级别审批
            setNextPMApprovalLevel(planApprovalTaskReq);
        }
        //审批通过，保存审批历史
        if (planApprovalTaskReq.getStatus() == 1) {
            //审批通过，更新task状态
            updateOnsiteTask(planApprovalTaskReq);
            preventMistakeService.setOnsiteTaskHis(planApprovalTaskReq.getTaskId());
        }
    }

    /**
     * 设置防错下一级别人员
     *
     * @param planApprovalTaskReq
     * @return boolean 是否有下一级别
     */
    private void setNextPMApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
        ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                .eq(ApprovalRule::getType, APPROVAL_TYPE_PM)
                .orderByDesc(ApprovalRule::getApprovalLevel)
                .last("limit 1"));
        //获取当前审批级别
        OnsiteApprovalTask onsiteApprovalTask = onsiteApprovalTaskMapper.selectOne(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
        int userLevel = onsiteApprovalTask.getApprovalLevel();
        if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
            ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
            onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                    .onsiteTaskId(onsiteApprovalTask.getOnsiteTaskId())
                    .approvalLevel(approvalUserRes.getApprovalLevel())
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0)
                    .build());
        } else {
            //如果是最后一个审批级别则设置task状态为不通过
            updateOnsiteTask(planApprovalTaskReq);
        }
    }

}
