package com.jihengbel.intelligent.applicationtask.init;

import com.jihengbel.intelligent.applicationtask.common.CommonUtil;
import com.jihengbel.intelligent.applicationtask.entry.FirstCalendar;
import com.jihengbel.intelligent.applicationtask.entry.OnsiteCalendar;
import com.jihengbel.intelligent.applicationtask.service.FirstTaskService;
import com.jihengbel.intelligent.applicationtask.service.OnsiteTaskService;
import com.jihengbel.intelligent.applicationtask.websocket.WsSessionManager;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.web.socket.TextMessage;

import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.FutureTask;

/**
 * onsite 提醒job
 */
@DisallowConcurrentExecution
public class FirstRemindJob implements Job {


    private int remind_handle_flag=1;//设置提醒为已经处理
    private int type=1 ;//首件
    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) jobExecutionContext.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    /**
     * 创建Task主方法
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String,Object> paramMap)  {
        Integer result=0;//没什么意义只是为了不出现整体并行
        List< FutureTask<Integer>> tasks=new ArrayList<>();
        tasks.add( getFirstFutureTask(paramMap,true));
        tasks.add( get200FutureTask(paramMap,true));
        tasks.add( getFirstFutureTask(paramMap,false));
        tasks.add( get200FutureTask(paramMap,false));
        try{
            for(FutureTask<Integer> t:tasks){
                t.get();
            }//等等所有线程结束
        }catch (Throwable e){
            e.printStackTrace();
        }

    }


    /**
     * first
     * @param paramMap
     * @param isWork
     * @return
     */
    private FutureTask<Integer> getFirstFutureTask(ConcurrentMap<String, Object> paramMap,boolean isWork) {
        HashMap param = new HashMap();
        param.put("type_flag",0);
        Callable cal=null;
        if(isWork){
            param.put("remind_task_flag",0);
            cal=new remindFirstWork(paramMap,param);
        }else{
            param.put("remind_nowork_flag",0);
            cal=new remindFirstNoWork(paramMap,param);
        }
        FutureTask<Integer> task = new FutureTask<Integer>(cal);
        Thread thread = new Thread(task);
        thread.start();  //开启线程
        return task;
    }


    /**
     * 200
     * @param paramMap
     * @param isWork
     * @return
     */
    private FutureTask<Integer> get200FutureTask(ConcurrentMap<String, Object> paramMap,boolean isWork) {
        HashMap param = new HashMap();
        param.put("type_flag",1);
        Callable cal=null;
        if(isWork){
            param.put("remind_task_flag",0);
            cal=new remindFirstWork(paramMap,param);
        }else{
            param.put("remind_nowork_flag",0);
            cal=new remindFirstNoWork(paramMap,param);
        }
        FutureTask<Integer> task = new FutureTask<Integer>(cal);
        Thread thread = new Thread(task);
        thread.start();  //开启线程
        return task;
    }

    /**
     * 发送提醒 需要工作的提醒
     */
    public class remindFirstWork implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap paramQuery;


        public remindFirstWork(ConcurrentMap<String, Object> paramMap, HashMap paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }




        @Override
        public Integer call() throws Exception {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            List<FirstCalendar> cals = firstTaskService.getRemindWork(paramQuery);
            String day = null;
            String format = "yyyy-MM-dd HH:mm";
            for (FirstCalendar cal : cals) {
                if (cal.getStart_day() != null && cal.getStart_day().trim().length() > 0) {
                    day = cal.getTask_time() + " " + cal.getStart_day();
                    format = "yyyy-MM-dd HH:mm";
                } else {
                    day = cal.getTask_time();
                    format = "yyyy-MM-dd";
                }
                boolean flag = CommonUtil.getDateCompare(CommonUtil.getStrDate(day, format, 0), new Date(),false);
                if (flag) {
                    String token =String.valueOf(cal.getUser_id());
                    if(WsSessionManager.SESSION_POOL.containsKey(token)){
                        try {
                            WsSessionManager.SESSION_POOL.get(token).
                                    sendMessage(new TextMessage("有一个首件巡检需要处理，请及时操作！"));
                            //update  onsite_task set ${flag}=${flagVal} where id=${onsite_task_id}
                            Map<String,Object> qm=new HashMap<>();
                            qm.put("flag","remind_task_flag");
                            qm.put("flagVal",remind_handle_flag);
                            qm.put("first_task_id",cal.getFirst_task_id());
                            firstTaskService.updateFirstRemindFlag(qm);
                        }catch (Throwable e){
                            e.printStackTrace();
                        }

                    }

                }

            }
            return 0;
        }
    }



    /**
     * 发送提醒  没有工作的提醒
     */
    public class remindFirstNoWork implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap paramQuery;


        public remindFirstNoWork(ConcurrentMap<String, Object> paramMap, HashMap paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }

        @Override
        public Integer call() throws Exception {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            List<FirstCalendar> cals = firstTaskService.getRemindWork(paramQuery);
            String day = null;
            String format = "yyyy-MM-dd HH:mm";
            int min;
            for (FirstCalendar cal : cals) {
                if (cal.getStart_day() != null && cal.getStart_day().trim().length() > 0) {
                    day = cal.getTask_time() + " " + cal.getStart_day();
                    format = "yyyy-MM-dd HH:mm";
                    min=cal.getDay_outtime()+cal.getDay_validate();
                } else {
                    day = cal.getTask_time()+ " " + cal.getTwo_hundred_submit_time();
                    format = "yyyy-MM-dd HH:mm";
                    min=0;
                }
                boolean flag = CommonUtil.getDateCompare(CommonUtil.getStrDate(day, format, min), new Date(),true);
                if (flag) {
                    Map map=new HashMap();
                    map.put("type",type);
                    Long  userid=firstTaskService.getRemindNoworkToManager(map);
                    if(userid!=null && userid>0){
                        map=new HashMap();
                        map.put("task_id",cal.getFirst_task_id());
                        map.put("approval_user_id",userid);
                        map.put("task_type",type);
                        map.put("flag","remind_nowork_flag");
                        map.put("flagVal",remind_handle_flag);
                        map.put("first_task_id",cal.getFirst_task_id());
                        firstTaskService.addRemindNowork(map);

                        //需要推动调用这个 todo
                    }
                }

            }
            return 0;
        }
    }

}
